"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const glob_1 = require("glob");
const vscode_languageserver_1 = require("vscode-languageserver");
const util_1 = require("util");
const fs = __importStar(require("fs-extra"));
const glob = util_1.promisify(glob_1.Glob);
const STATIC_RESOURCE_DECLARATION_FILE = '.sfdx/typings/lwc/staticresources.d.ts';
const STATIC_RESOURCE_INDEX_FILE = '.sfdx/indexes/lwc/staticresources.json';
let STATIC_RESOURCES = new Set();
function resetStaticResources() {
    STATIC_RESOURCES.clear();
}
exports.resetStaticResources = resetStaticResources;
function getResourceName(resourceMetaFile) {
    const resourceFile = resourceMetaFile.substring(0, resourceMetaFile.lastIndexOf('-'));
    return path_1.parse(resourceFile).name;
}
async function updateStaticResourceIndex(updates, { workspaceRoots }, writeConfigs = true) {
    let didChange = false;
    for (const update of updates) {
        if (update.uri.endsWith('.resource-meta.xml')) {
            const resourceName = getResourceName(update.uri);
            switch (update.type) {
                case vscode_languageserver_1.FileChangeType.Created:
                    didChange = true;
                    STATIC_RESOURCES.add(resourceName);
                case vscode_languageserver_1.FileChangeType.Deleted:
                    STATIC_RESOURCES.delete(resourceName);
                    didChange = true;
            }
        }
        if (didChange) {
            return processStaticResources(workspaceRoots[0], writeConfigs);
        }
    }
}
exports.updateStaticResourceIndex = updateStaticResourceIndex;
async function processStaticResources(workspace, writeConfigs) {
    if (STATIC_RESOURCES.size > 0 && writeConfigs) {
        const filename = path_1.join(workspace, STATIC_RESOURCE_DECLARATION_FILE);
        const fileContent = generateResourceTypeDeclarations();
        return fs.writeFile(filename, fileContent);
    }
}
async function indexStaticResources(context, writeConfigs = true) {
    const { workspaceRoots } = context;
    const { sfdxPackageDirsPattern } = await context.getSfdxProjectConfig();
    const STATIC_RESOURCE_GLOB_PATTERN = `${sfdxPackageDirsPattern}/**/staticresources/*.resource-meta.xml`;
    const workspace = workspaceRoots[0];
    try {
        if (initStaticResourceIndex(workspace)) {
            return Promise.resolve();
        }
        else {
            const files = await glob(STATIC_RESOURCE_GLOB_PATTERN, { cwd: workspaceRoots[0] });
            for (const file of files) {
                STATIC_RESOURCES.add(getResourceName(file));
            }
            return processStaticResources(workspace, writeConfigs);
        }
    }
    catch (err) {
        console.log(`Error queuing up indexing of static resources. Error details:`, err);
        throw err;
    }
}
exports.indexStaticResources = indexStaticResources;
function generateResourceTypeDeclarations() {
    return Array.from(STATIC_RESOURCES)
        .sort()
        .map(resourceDeclaration)
        .join('');
}
function resourceDeclaration(resourceName) {
    return `declare module "@salesforce/resourceUrl/${resourceName}" {
    var ${resourceName}: string;
    export default ${resourceName};
}
`;
}
function initStaticResourceIndex(workspace) {
    const indexPath = path_1.join(workspace, STATIC_RESOURCE_INDEX_FILE);
    const shouldInit = fs.existsSync(indexPath);
    if (shouldInit) {
        const indexJsonString = fs.readFileSync(indexPath, 'utf8');
        const staticIndex = JSON.parse(indexJsonString);
        STATIC_RESOURCES = new Set(staticIndex);
        return STATIC_RESOURCES;
    }
}
function persistStaticResources(context) {
    const { workspaceRoots } = context;
    const indexPath = path_1.join(workspaceRoots[0], STATIC_RESOURCE_INDEX_FILE);
    const index = Array.from(STATIC_RESOURCES);
    const indexJsonString = JSON.stringify(index);
    fs.writeFile(indexPath, indexJsonString);
}
exports.persistStaticResources = persistStaticResources;
//# sourceMappingURL=static-resources-util.js.map